unit DS036eu; //"eu" in name to imply "external unit"

//EDIT THIS!!!!!!!!!!!!!!!!!!!!!!!!
//GO BACK TO earlier DS's using DS036eu... upgrade to aca, and incorp version checks.

(*To provide some commonly needed routines for working with
Dallas 1-Wire (aka MicroLan) chips.*)

(*If you are not TKB, you can skip over the "To Do"s section*)

(*TO DO starts here>>>>>>>>>>>>>>

  SORT AVAIL CALLS INTO LOGICAL GROUPS(?)

  REMOVE BAD ONE- TMROM?

  CONSIDER PROBLEM in NextAndRomRead... if there IS NO "next", do you want
    to read? Probably should do new, better NextAndRomRead with "RomRead" conditional
    on "Next" finding something, and a message to go back from module?
    DON'T re-write current... keep, for eventual sloughing.

(*DONE? Easy to do if not... Provide two levels of "read chip": One WITHOUT session set up, chip
  isolated, and another (wrapping previous) with those two things done.*)

(*Some serious work on variable names probably wise.
  Ditto ssTKBErr error codes returned.
  Ditto version conventions...*)

(*<<<<< TO DO ends here*)

(*Be sure you understand issues of "scope". For a while, at least,
there may be cases in my code where there is, for example,
a case of liDMLSHandle in the code calling routines from this
external unit AND a DIFFERENT instance of liDMLSHandle within this code.*)

(*When working with this unit, don't pay too much attention to the 2nd,
3rd, etc error messages you receive when you run your code....
Fix whatever the first problem is, and subsequent ones may go away.*)

(*It may seem that some or all of the following are pointless
complications of the Dallas TMEX. However, they offer....
A consistent "no error found" flag (ssTKBErr[0]=chr(0)),
A way to determine where an error occurred. (Non-null ssTKBErr values.)
A (perhaps) tidier implementation of a marvelously subtle system *)

(*=== HERE BEGINS IMPORTANT LESSON IN POINTERS....
From Delphi helpfile, pointers reference:
"You can use the @ (address-of) operator and the function Addr
to create pointer values that are treated as pointers to
dynamic variables."

Calling program will need two buffer areas, typically called
siaROM and baMLStateBuffer. They must be of the types
Tsia8 and TbaDMLStateBuffer respectively. Happily, they are
both defined in this (DS036eu) unit, so the calling program only
needs to declare...

    baDMLStateBuffer:TbaDMLStateBuffer;
    siaDMLRom:Tsia8;

(In addition to having DS036eu in the uses clause, and a copy
 of it (or it's .dcu) in the calling program's folder (or
 otherwise available) during the compile time. You still end
 up with just one .exe to deply.)

Several calls to this unit need the addresses of the starts
of those buffers to be passed to the routine. For example,
in...

procedure DMLFirst(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:string;
          var iDalErr:integer);
begin
iDalErr:=TMFirst(liSessHandle,@addrOfbaDMLStateBuffer);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu14');
  end;
end;//DMLTMRom

... the second parameter is an example. Note that the "addr"
   in the name is purely for human convenience. The variable
   could be called "v1". ALSO NOTE that the parameter is
   passed as a "var" type call. THIS is why the system
   passes THE ADDRESS OF the passed variable to
   the called routine... which is why the call of TMFirst
   within the routine uses the @ sign. Because of that @
   sign, the call of TMFirst passes THE ADDRESS of the
   state buffer... which is what TMFirst needs!

Hang in there... nearly done.

Now, in DS036eu, its call of the underlying TMEX routine
looks like...

DMLFirst(liASessionHandle,baDMLStateBufferAStateBuffer,
             ssTKBErrReturned,iDalErrReturned);

NOTE: THERE IS ONLY ONE '@' sign in all of this, calling
code and handling code. You may see other '@'s in other
code, and/or variables declared as pointers, or use of
the caret (^), which is also connected with pointers, but
the magic VAR used in defining the procedure as I have
makes things NICE AND SIMPLE!!

Ummm... it wasn't always thus... apologies if you stumble
across bits of code written by me doing things
other, less simple ways!

(*===== HERE ENDS POINTERS STUFF=====*)


(*  Naming of Data types....

(If "data types" doesn't mean something
very specific, and perhaps a little esoteric, see
http://sheepdogguides.com/dst1a.htm
)

From SDK help file: (except first column)...

    Delphi name Dallas name   Desc
    integer     short         16-bit signed
    longint     long          32-bit signed
    shortint    char          8-bit signed
    byte        unsigned char 8-bit unsigned

    ??          far pascal		far function using pascal calling convention
    ??          void far *		generic far pointer
    ??          short far *		far pointer to a 16 bit signed integer

*)


interface

uses dialogs,sysutils,IBTMEXPW;

const ver='aca 5 Mar 05';

(*First 3 characters: aaa for earliest version. aab would be one which
   should work better than aaa, but not require any changes to the calling
   program, i.e. it should still support everything aaa supported. If the
   middle or 1st character changes, programs written for them are not
   guaranteed to work with with earlier versions of the external unit.
   Typically, middle letter will change when a new routine is added,
   or if parameters passed to any routine change.

   >>>> THESE IDEAS ON VERSION conventions need sharpening up!*)

(*Features.... newest at top....

At change from aba to aca...
   Any strings exhanged with caller became exchanged in a shortstring,
      so that DLLs could have DS036eu in their uses clauses.
      This will require editing of external references to
      DS036eu routines using strings.

   Added function ssExtendShortString
   Added constant iSizeOfShortStringBuff;

At change from aaa to aba...
   DMLReadDSEeprom gained another parameter: iBytesToRead
     (no need to keep "aaa"... anything written with old version
      will not compile, but any calls to DMLReadDSEeprom can easily be fixed.)

(*My first "serious" unit. Written same day as first unit, 21 Feb 05*)
(*Written at 7NC, inspired by work for uni student doing stock control
  via eeprom ibuttons.*)

(*Error -201 can arise when a call is made with a useless value in the
  session handle.*)

iSizeOfShortStringBuff=256;(*Sets limit on size of messages which can be sent
   from unit*)


type Tba4k=array [0..511] of byte;(*Name derives from Dallas practice of
                       describing 0.5kbyte eeprom as a 4kbit device*)
     Tsia8=array[0..8] of smallint;(*Yes: 9 bytes.
                 the last isn't used every time, but I think
                 it IS used sometimes.
                 Odd: Smallint is a 16 bit data-type. One
                 would have thought byte the right type,
                 but small int WORKING, widely...
                 Dbl checked 2/05... yes, TMEX is
                 using 16 bits for each 8 bits of ROM id.
                 (see TMEX help, code example for TMRom*)
     TbaDMLStateBuffer=array[0..15361] of byte;
                 (*15360 might do, but for one byte, I covered
                 the possiblilty that I am misunderstanding
                 something in the Dallas docs*)

var      sTmp:string;

function DS036Version:string;

procedure DMLStrongAccess(
          liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          (*N.B. from Delphi helpfile:
          The compiler performs no range checks when indexing a
          character pointer because it has no type information
          available to determine the maximum length of the
          null-terminated string pointed to by the character
          pointer.
          >>>>>>>>Your program must perform any such range checking.

          ???Can the user over-run the space set aside for the pchar???*)

          var iDalErr:integer);
(* Skeleton for call....
if ssTKBErrRet='' then begin
DMLStrongAccess(liDML,ba,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLExtStartAndSetup(
          var liSessHandle:longint;
          iDMLPortNum,iDMLAdapterType:integer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLExtStartAndSetup(liDML,iDML,iDML,ssTKBErrRet,iDalErrRet); //<<Do more of these skeltons
end;
*)

procedure DMLEndSession(
          liSessHandle:longint;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLEndSession(liDML,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLFirstAndTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLFirstAndTMRomRead(liDML,ba,sia,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLNextAndTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLNextAndTMRomRead(liDML,ba,sia,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*See notes in implementation... 1st byte in siaRom is made 0
     in early part of this routine
if ssTKBErrRet='' then begin
DMLTMRomRead(liDML,ba,sia,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLTMRomWrite(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLTMRomWrite(liDML,ba,sia,ssTKBErrRet,iDalErrRet);
end;
*)

(*Type is just for NOT YET WORKING sDMLChipID
  NOT LEAST: Calling mechanism probably needs fixing for Var / @ / ^ issues*)
type TptrStr=^string;
function sDMLChipID(addrOfsiaROM:TptrStr):string;

procedure DMLFirst(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*This MAY be useful... but use DMLFirstAndRomRead where appropriate
if ssTKBErrRet='' then begin
DMLFirst(li,ba,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLNext(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*This MAY be useful... but create and use DMLNextAndRomRead where appropriate
  It will sometimes be necessary to split Next and RomRead to check whether
  Next returned a "no more chips seen" response.... indeed it may pay to
  create a new, better NextAndRom with extra returned value to say "ran out
  of chips, didn't do Rom before returning".
if ssTKBErrRet='' then begin
DMLNext(li,ba,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLReadDSEeprom(liDMLSessHdl:longint;
         var baDataArr:Tba4k;
         iBytesToRead:integer;
         var ssTKBErr:shortstring;
         var iDalErr:integer);
(*Note data type of baDataArray. Defined in DS036eu
if ssTKBErrRet='' then begin
DMLReadDSEeprom(li,ba,ssTKBErrRet,iDalErrRet);
end;
*)

procedure DMLTouchReset(
          liSessHandle:longint;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*
if ssTKBErrRet='' then begin
DMLTouchReset(liDML,ssTKBErrRet,iDalErrRet);
end;
*)

function sRepOfTKBAndDal(sErr:string;iErr:integer):string;

function ssStrToSS(sTmp:string):shortstring;
(*Must not be used from outside DS036eu. If it is, DLLs with "uses DS036eu"
  will mess up, and mess up in a very tedious way.*)

{============================================}
implementation

function DS036Version:string;
(*Part of DS036eu since ver aaa.
  Reports version of DS036eu. See notes elsewhere about what
  can be known from the letters in the ID*)
begin
result:=ver;
end; //DS036Version

procedure DMLStrongAccess(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*To quote from Dallas: "The TMStrongAccess API call
verifies that the device in the ROM buffer is on the
MicroLan and selects it for a device specific command.
This is the same as TMAccess except instead of using
a match ROM command the search ROM command is used.
This only disadvantage to this API call is that it
takes three times as long to run as TMAccess.".. and,
also quoting Dallas:
"  0 => device in ROM buffer not on MicroLan
   1 => device in ROM buffer on MicroLan and it is selected
  <0 => TMEX Network Error Return Code
"*)
begin
ssTKBErr[0]:=chr(0);
  iDalErr:=TMStrongAccess(liSessHandle,@addrOfbaDMLStateBuffer);
  if iDalErr<>1 then begin
    ssTKBErr:=ssStrToSS('D36eu01');
    end;

{Could? Should? CheckSess occur here?}

end;//DMLStrongAccess

procedure DMLExtStartAndSetup(
          var liSessHandle:longint;
          iDMLPortNum,iDMLAdapterType:integer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since ver aaa.
(*This procedure prepares the way for further ROM Function Commands.

Does an TMExtendedStartSession to initiate session.
Then does a TMSetup, as that's always needed, and
       to see Start succeeded.

As soon as the session is no longer needed, a call of
TMEndSession should be made.*)

(*In some Dallas demo code, there seems to be
  places where the code says
  "Couldn't do that first time? Try a few times more"
  In this code, we only try once.
  It is up to the calling program to
  re-call this as often as it deems worthwhile.*)

var c1:byte;
begin
ssTKBErr[0]:=chr(0);

liSessHandle:=TMExtendedStartSession(iDMLPortNum,
                 iDMLAdapterType,nil);
if liSessHandle<1 then begin
  ssTKBErr:=ssStrToSS('D36eu7');
  iDalErr:=liSessHandle;
  end;

if ssTKBErr[0]=chr(0) then begin
  iDalErr:=TMSetUp(liSessHandle);
  if iDalErr<>1 then begin
    sTmp:='D36eu8';
    if iDalErr=3 then
      sTmp:=sTmp+'- MicroLan does not exist.';
    ssTKBErr:=ssStrToSS(sTmp);
    end;
  end;
end;//DMLExtStartAndSetup

procedure DMLEndSession(
          liSessHandle:longint;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since ver aaa.
(*This procedure closes the "connection" between the user's
 program and the TMEX drivers.
 It also releases the handle for other use.*)

begin
iDalErr:=TMEndSession(liSessHandle);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu9');
  end;
(*From Dallas Docs:  Return Value
   0   => session_handle already invalid
   1   => session ended, session_handle no longer valid
  <0  => TMEX Session Error Return Codes*)
end;//DMLTMEndSession

procedure DMLFirstAndTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since aaa.
  Before calling this, call DMLExtStartAndSetup.
  DMLFirstAndRomRead looks at Lan, tries to find a chip. If it finds
  one, the chip's ID ("ROM") will be returned to the calling
  program after this call.
  Supply DMLNextAndRomRead with session handle which was
  returned by DMLExtStartAndSetup. Other two params
  are for return of info only.

  To quote from Dallas documentation...

  "The TMRom API call transfers a ROM data pattern between
  the internal eight-byte buffer maintained by the driver
  and an array 'ROM' of eight short integers declared in the
  application program. The direction of data transfer is
  specified by the value of the first integer in the array.
  If the first integer is zero, then the eight bytes from
  the internal buffer are transferred into the eight
  integer variables of the integer array. If the first
  integer is non-zero, then the least significant bytes
  of the eight integers are transferred into the internal
  eight-byte buffer."

  The TMRomRead call is merely so that the ID of the device
  which was accessed can be known by the calling program.

  DON'T GET CONFUSED by things arising because this DS036eu
  procedure itself calls Ds036eu routines....

*)
begin
DMLFirst(liSessHandle,
addrOfbaDMLStateBuffer,//NO @ on this one... @ only if not ALREADY passed by var
    //This is different from most things in DS036eu because here we have
    //something in DS036eu calling something else in DS036eu. When calls
    //are made FROM THE OUTSIDE to things in DS036eu, then an '@' will sometimes
    //be needed.
    ssTKBErr,iDalErr);
if ssTKBErr[0]<>chr(0) then ssTKBErr:=ssStrToSS(ssTKBErr+'-A In FirstAndRomRead');
if ssTKBErr[0]=chr(0) then begin
       addrOfsiaRom[0]:=0;
       DMLTMRomRead(liSessHandle,addrOfbaDMLStateBuffer,
              addrOfsiaROM,ssTKBErr,iDalErr);
             //see comment above about absense of '@'

       if ssTKBErr[0]<>chr(0) then ssTKBErr:=ssStrToSS(ssTKBErr+'-B In FirstAndRomRead');
       end;
end; //DMLFirstAndTMRomRead

procedure DMLNextAndTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since aaa.
  Before calling this, call DMLExtStartAndSetup, AND
  TMFirst. (You will probably do that via DMLFirstAndRomRead
  as you are using this external unit. (DS036eu))
  DMLNextAndRomRead looks at Lan, tries to find another chip. If it finds
  one, the chip's ID ("ROM") will be returned to the calling
  program after this call.
  Supply DMLNextAndRomRead with session handle which was
  returned by DMLExtStartAndSetup. Other two params
  are for return of info only.

  To quote from Dallas documentation...

  "The TMRom API call transfers a ROM data pattern between
  the internal eight-byte buffer maintained by the driver
  and an array 'ROM' of eight short integers declared in the
  application program. The direction of data transfer is
  specified by the value of the first integer in the array.
  If the first integer is zero, then the eight bytes from
  the internal buffer are transferred into the eight
  integer variables of the integer array. If the first
  integer is non-zero, then the least significant bytes
  of the eight integers are transferred into the internal
  eight-byte buffer."

  The TMRomRead call is merely so that the ID of the device
  which was accessed can be known by the calling program.

  DON'T GET CONFUSED by things arising because this DS036eu
  procedure itself calls Ds036eu routines....

*)
begin
DMLNext(liSessHandle,
addrOfbaDMLStateBuffer,//NO @ on this one... @ only if not ALREADY passed by var
    //This is different from most things in DS036eu because here we have
    //something in DS036eu calling something else in DS036eu. When calls
    //are made FROM THE OUTSIDE to things in DS036eu, then an '@' will sometimes
    //be needed.
ssTKBErr,iDalErr);
if ssTKBErr[0]<>chr(0) then ssTKBErr:=ssStrToSS(ssTKBErr+'-A In NextAndRomRead');
if ssTKBErr[0]=chr(0) then begin
       addrOfsiaRom[0]:=0;//to cause TMRom to "read"
       DMLTMRomRead(liSessHandle,addrOfbaDMLStateBuffer,
                                addrOfsiaROM,ssTKBErr,iDalErr);
       if ssTKBErr[0]<>chr(0) then ssTKBErr:=ssStrToSS(ssTKBErr+'-B In NextAndRomRead');
       end;
end; //DMLNextAndTMRomRead

function sDMLChipID(addrOfsiaROM:TptrStr):string;
(*SEE NOTES IN FORWARD DECL
  Part of DS036eu since ver aaa.... but not working in aaa at one stage
  Returns a string giving the chip's address.
  Calling program must have siaROM buffer. (See DS036eu preface)
  Before calling: Stuff a chip ID ("ROM") into the siaROM buffer.
  Returns string in format as displayed by iButtonViewer.
  If calling prgm calls that siaCallerROM, correct call of
  this routine from calling prgm is sDMLChipID(@siaCallerROM),
  i.e.: "pass the address of the buffer to the routine"

  "^integer" cannot be used in function heading.
  merely declares addrOfsiaROM "lies", saying that
  addrOfsiaROM holds the address where an integer is stored.
  Lying thus allows us to access the value there.

  There's help on pointers in Delphi at...
    delphi.about.com/library/rtl/blrtlPtr.htm

  THE POINTER STUFF ABOVE WRITTEN BEFORE I UNDERSTOOD BETTER, WHICH
  HAPPENED IN LATE FEB 05
    *)
var c1:byte;
    c2:integer;
    ptrAddr:TptrStr;
    sTmp,sTmp2:string[49];
begin
ptrAddr:=addrOfsiaROM;
sTmp:=ptrAddr^;
//sTmp:='ABCDE';
sTmp2:=inttohex(ord(sTmp[0]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[1]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[2]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[3]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[4]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[5]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[6]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[7]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[8]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[9]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[10]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[11]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[12]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[13]),2);sTmp2:=sTmp2+' '+inttohex(ord(sTmp[14]),2);
sTmp2:=sTmp2+' '+inttohex(ord(sTmp[15]),2);
//sTmp:=inttohex(c2,2);
//sTmp2:=inttohex(255,2);

result:='sDMLChipID in DS036eu doesn''t work yet!';//inttohex(c2,2);
end; //sDMLChipID

procedure DMLFirst(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since aaa.
  Before calling this, call DMLExtStartAndSetup.
  This looks at Lan, tries to find a chip. If it finds
  one, the chip's ID ("ROM") will be in siaDMLRom
  after this call. Supply DMLFirst with session handle
  which was returned by DMLExtStartAndSetup. Other
  two params are for return of info only. *)

begin
iDalErr:=TMFirst(liSessHandle,@addrOfbaDMLStateBuffer);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu14');
  end;
end;//DMLTMRom

{From Dallas:
The TMFirst API call finds the first multi-drop device
on the MicroLan specified with the session_handle.
A ROM search algorithm is used to find the first
unique serial registration number (ROM) data pattern
on the MicroLan.  The ROM data pattern that was found
is stored in an internal eight byte buffer. The internal
eight byte buffer can be read using the TMRom API call.
TKB: N.B.: that "internal 8 byte buffer" is NOT the buffer
I usually call siaDMLRom

short far pascal TMFirst(

long session_handle, 	// session handle for the desired MicroLan
void far *state_buffer	// state buffer provided for the MicroLan session


Return Value

0   => device not found
1   => first device on the MicroLan found
<0 => TMEX Network Error Return Code}

procedure DMLNext(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since aaa.
  Before calling this, call DMLExtStartAndSetup, and
  call DMLFirst.
  This looks at Lan, tries to find a chip. If it finds
  one, the chip's ID ("ROM") will be in siaDMLRom
  after this call. Supply DMLNext with session handle
  which was returned by DMLExtStartAndSetup. Other
  two params are for return of info only. *)

begin
iDalErr:=TMNext(liSessHandle,@addrOfbaDMLStateBuffer);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu16');
  end;
end;//DMLNext
{See notes with DMLFirst}

procedure DMLTMRomRead(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since aaa.
  Before calling this, call at least DMLExtStartAndSetup,

  DMLTMRomRead looks at TMEX's internal buffer, moves chip's ID ("ROM")
  to "siaROM", or whatever it is called, buffer.....

  Supply DMLRomRead with session handle which
  was returned by DMLExtStartAndSetup, and addr of the
  StateBuffer it is using. Other three
  params are only for data's return. *)

begin
addrOfsiaRom[0]:=0;
iDalErr:=TMROM(liSessHandle,@addrOfbaDMLStateBuffer,@addrOfsiaRom);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu12');
  end;
end;//DMLTMRomRead

procedure DMLTMRomWrite(liSessHandle:longint;
          var addrOfbaDMLStateBuffer:TbaDMLStateBuffer;
          var addrOfsiaROM:Tsia8;
          var ssTKBErr:shortstring;
          var iDalErr:integer);

(*Part of DS036eu since aaa.
  Before calling this, call at least DMLExtStartAndSetup, and
  put something meaningful in the user's programs siaRom buffer.
  That "someting meaningful" should not have 0 in the first
  (i.e. chip family) byte. (See DMLTMRomRead.)

  DMLTMRomWrite moves what's in the buffer "siaROM", or whatever
  it is called, to an internal buffer of TMEX.

  Supply DMLRomRead with session handle which
  was returned by DMLExtStartAndSetup, addr of the
  StateBuffer it is using, and addr of the buffer
  with the chip id in it. Other two
  params are only for return of info. *)

begin
iDalErr:=TMROM(liSessHandle,@addrOfbaDMLStateBuffer,@addrOfsiaRom);
if iDalErr<>1 then begin
  ssTKBErr:=ssStrToSS('D36eu17');
  end;
end;//DMLRMRomWrite

procedure DMLReadDSEeprom(liDMLSessHdl:longint;
         var baDataArr:Tba4k;
         iBytesToRead:integer;
         var ssTKBErr:shortstring;
         var iDalErr:integer);

(*Part of DS036eu since aaa.
  Calling program needs a "[0..511] of byte" array. In order to
  be able to pass that, use the type called Tba4k defined
  in this unit when you declare (in the "outer" program) the array.

  Only the first iBytesToRead of the array will be used,
  as only the first iBytesToRead of the eeprom are read...
     but the array might as well be capable of any read.

  If a ReadDSEeprom which starts somewhere other than with
  the first byte is wanted, create a new proc, don't revise
  this one.

  Before calling this, call DMLExtStartAndSetup, and do
  something (perhaps TMFirst) to fetch a chip's ID ("ROM"),
  (or establish the "ROM" in the relevant place "by hand".
  Keep doing that 'til a suitable chip (family $23 or $06)
  is found. Then call TMRom. And now you are ready for this!
  This will transfer 512 bytes from inside the EEPROM to the
  array.

  Things outside DMLReadDSEeprom should
     i) reset MicroLan and
     ii) relinquish session handle
     not long after DMLReadDSEeProm
     (Can be dine with calls of <???> and DMLEndSession)

*)

(*Access the chip, and fill baDataArr (and calling array) with
  the first iBytesToRead from the eeprom.*)
(*Has a "fake read" alternate routine built in, with hardcoded
  switch near start of routine's code*)

var bFromEeprom:byte;
    //vvvvvvvv RENAME to reflect type
    c1,c2:integer;
    iOffset:integer;
    ptrToSia:TptrStr;//type TptrStr declared in DS036eu
    boCreateFakeData:boolean;
begin
boCreateFakeData:= false; //set true for test data, false for real read
    // setting true may result in things that no longer work with
    //otherwise ok DS037
if boCreateFakeData then begin
  c1:=0;
  repeat
     c2:=0;
     repeat
       baDataArr[(c1*32)+c2]:=65+c1;
       inc(c2)
     until (c2=32) or ((c1*32)+c2>iBytesToRead);
     inc(c1);
  until (c1=16) or (((c1-1)*32)+c2>iBytesToRead)
 end //no ; End of "produce fake data" routine.

else begin //real read from chip
{Start of read eeprom contents. Before here, specific chip must be selected.}
{Issue (i.e. TX, or transmit) Memory Command "Read Memory", F0h}
if ssTKBErr[0]=chr(0) then begin
  iDalErr:=TMTouchByte(liDMLSessHdl,$F0);
  if iDalErr<0 then begin
     ssTKBErr:=ssStrToSS('REeprom0');
     showmessage('DMLReadDSEeprom encountered error '+
       'during 1st call of TMTouchByte:'+sRepOfTKBAndDal(ssTKBErr,iDalErr));
     if iDalErr=-1 then showmessage('There may be short on ML');
    end;
end;
{Could? Should? Check Sess occur here?
if bTKBErr=0 (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liSessHandle);
if iDMLTmpResult then begin
    bTKBErr:=62;
    iDalErr:=iDMLTmpResult;
    end;
  end;
}

{TX TA1, LSByte of beginning offset. 0 to read whole memory}
if ssTKBErr[0]=chr(0) then begin
  iDalErr:=TMTouchByte(liDMLSessHdl,$00);
  if iDalErr<0 then begin
    ssTKBErr:=ssStrToSS('REeprom1');//redo this and following to match previous
    if iDalErr=-1 then ssTKBErr:=ssStrToSS(ssTKBErr+'- Likely: shorted ML');
    end;
end;
{Could? Should? Check Sess occur here?
if bTKBErr=0 (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liSessHandle);
if iDMLTmpResult then begin
    bTKBErr:=62;
    iDalErr:=iDMLTmpResult;
    end;
  end;
}

{TX TA2, MSByte of beginning offset. 0 to read whole memory}
if ssTKBErr[0]=chr(0) then begin
  iDalErr:=TMTouchByte(liDMLSessHdl,$00);
  if iDalErr<0 then begin
    ssTKBErr:=ssStrToSS('REeprom2');//redo this and following to match previous
    if iDalErr=-1 then ssTKBErr:=ssStrToSS(ssTKBErr+'- Likely: shorted ML');
    end;
end;
{Could? Should? Check Sess occur here?
if bTKBErr=0 (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liSessHandle);
if iDMLTmpResult then begin
    bTKBErr:=62;
    iDalErr:=iDMLTmpResult;
    end;
  end;
}
//DSEeprom, e.g. DS2433, will now xmit 512 bytes over MicroLan to the master.
//Maybe init array with "background" and make first bytes say "Not Read" ?
for iOffset:=0 to iBytesToRead do begin
{Start of bad indenting...
 Here begins pick up next byte from MicroLan, put in bFromEeprom}
  if ssTKBErr[0]=chr(0) then begin
  bFromEeprom:=255;//In case read from MicroLan fails
  iDalErr:=TMTouchByte(liDMLSessHdl,$FF);
  (*iDal"Err" will be holding good data if TouchByte proceeded ok.*)
  (*Doesn't quite make sense to me (yet!), but fwiw...
  From TMEX HELP:
  Return Value
    >=0 => byte returned from MicroLan while sending byte
    0xFF => 1-wire bus is shorted.
    <0 => TMEX Hardware Specific Error Return Code
  My question: How does chip send a $FF to the master? How do we tell
     "it held FF" and "Lan was shorted"?
     ... maybe:
        "it held FF" is $00FF, and
        "ML was shorted" is $0xFF **WHERE X<>0** ?

  *)
  if iDalErr<0 then begin(*-1 flags shorted ML*)
    ssTKBErr:=ssStrToSS('REeprom4');
    end;
  if iDalErr>255 then begin
    ssTKBErr:=ssStrToSS('REeprom5');
    end;
  if (iDalErr>=0)and(iDalErr<256) then begin
    bFromEeprom:=iDalErr;//Yes... use intermediary in case Check Sess used.
{Scraps from DS034
    if bByteReturned<>bConfirmation then begin
         ssTKBErr:=ssStrToSS('REeprom6: Conf byte from 2409 didn''t match '+
         'Status Info Byte');
         iDalErr:=iDMLTmpResult;(*-1 flags shorted ML*)
         end;}
    end;
end;
{Could? Should? Check Sess occur here?
if bTKBErr=0 (*CSess*) then begin
        iDMLTmpResult:=TMValidSession(liSessHandle);
if iDMLTmpResult then begin
    bTKBErr:=62;
    iDalErr:=iDMLTmpResult;
    end;
  end;
}

{... end of bad indenting}

//bFromEeprom:=bOffsetPage;//debug aid

     {By now, byte sent from EEPROM is in bFromEeprom, if all well.}
     {Try to add some "Did it read okay?" code.}
     {Explain for self and tut why stream of bytes from EEPROM is
         sychronized with DS034's reading of those bytes.}

baDataArr[iOffset]:=bFromEeprom;
end;//for c1:=0 to 511
end;//.. else real read from chip

(*Things outside DMLReadDSEeprom should now reset MicroLan and
  relinquish session handle. (I.e. they should call
  DMLTouchReset and DMLEndSession*)

if boCreateFakeData then ssTKBErr:=ssStrToSS('FakedData');//Don't alter string- used as flag
end; //DMLReadDSEeprom

procedure DMLTouchReset(
          liSessHandle:longint;
          var ssTKBErr:shortstring;
          var iDalErr:integer);
(*Part of DS036eu since ver aaa.
(*This procedure "reawakens" chips on MicroLan
  which had been "asleep" after, say, a TMStrongAccess.*)

begin
iDalErr:=TMTouchReset(liSessHandle);
if (iDalErr<>1)and(iDalErr<>2) then begin
  ssTKBErr:=ssStrToSS('D36eu15');
  end;
(*From Dallas Docs:  Return Values
   0   => No presence pulse detected.
   1   => Non-alarming presence pulse detected.
   2   => Alarming presence pulse detected. (not available on all platforms)

   3   => MicroLan is shorted.

   5   => TMTouchReset not supported
  <0   => TMEX Hardware Specific Error Return Codes*)
end;//DMLTouchReset

function sRepOfTKBAndDal(sErr:string;iErr:integer):string;
(*Part of DS036eu since ver aaa.
(*For error reporting*)
begin
result:='String: '+sErr+' Dallas: '+inttostr(iErr);
end; // sRepOfTKBAndDal

function ssStrToSS(sTmp:string):shortstring;
(*May not be necessary... left over from failed attempt to use pchar
  to make DLLL friendly*)
(*Must not be used from outside DS036eu? If it is, DLLs with "uses DS036eu"
  will mess up, and mess up in a very tedious way.*)

(*Could this be made better?*)
var c1,c2:integer;
    ssTmp:shortstring;
//    caBuff:array[0..iSizeOfShortStringBuff] of char;
    (*Could be done with constant, but that would make changing the
      size of the buffer more error prone*)
begin
c2:=length(sTmp);
if c2>255-7 then
    sTmp:=copy(sTmp,1,c2-7)+'CLIPPED';
result:=sTmp;
end;//ssStrToSS

end.
